@@ -1,12 +1,12 @@
-{-# LANGUAGE TypeFamilies
-           , MultiParamTypeClasses
-           , FlexibleInstances
-           , FlexibleContexts
-           , ExistentialQuantification
-           , TypeSynonymInstances
-           , DeriveDataTypeable
-           , ViewPatterns
-  #-}
+{-# LANGUAGE DeriveDataTypeable        #-}
+{-# LANGUAGE ExistentialQuantification #-}
+{-# LANGUAGE FlexibleContexts          #-}
+{-# LANGUAGE FlexibleInstances         #-}
+{-# LANGUAGE GADTs                     #-}
+{-# LANGUAGE MultiParamTypeClasses     #-}
+{-# LANGUAGE TypeFamilies              #-}
+{-# LANGUAGE TypeSynonymInstances      #-}
+{-# LANGUAGE ViewPatterns              #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -29,34 +29,41 @@
 -- sometimes to work around an apparent bug in certain versions of
 -- GHC, which results in a \"not in scope\" error for 'CairoOptions'.
 --
+-- The types of all the @fromX@ functions look funny in the Haddock
+-- output, which displays them like @Type -> Type@.  In fact they are
+-- all of the form @Type -> Graphics.Rendering.Cairo.Type@, /i.e./
+-- they convert from a diagrams type to a cairo type of the same name.
 -----------------------------------------------------------------------------
 module Diagrams.Backend.Cairo.Internal where
 
-import Diagrams.Core.Transform
+import           Diagrams.Core.Transform
 
-import Diagrams.Prelude
-import Diagrams.TwoD.Path (Clip(..), getFillRule)
-import Diagrams.TwoD.Text
-import Diagrams.TwoD.Image
-import Diagrams.TwoD.Adjust (adjustDia2D, setDefault2DAttributes)
-import Diagrams.TwoD.Size (requiredScaleT)
+import           Diagrams.Located                (viewLoc)
+import           Diagrams.Prelude
+import           Diagrams.Trail
+import           Diagrams.TwoD.Adjust            (adjustDia2D,
+                                                  setDefault2DAttributes)
+import           Diagrams.TwoD.Image
+import           Diagrams.TwoD.Path              (Clip (..), getFillRule)
+import           Diagrams.TwoD.Size              (requiredScaleT)
+import           Diagrams.TwoD.Text
 
-import qualified Graphics.Rendering.Cairo as C
+import qualified Graphics.Rendering.Cairo        as C
 import qualified Graphics.Rendering.Cairo.Matrix as CM
 
-import Control.Monad.State
-import Data.Maybe (catMaybes, fromMaybe)
-import Data.List (isSuffixOf)
+import           Control.Monad.State
+import           Data.List                       (isSuffixOf)
+import           Data.Maybe                      (catMaybes, fromMaybe)
 
-import Control.Exception (try)
+import           Control.Exception               (try)
 
-import qualified Data.Foldable as F
+import qualified Data.Foldable                   as F
 
-import Data.Typeable
+import           Data.Typeable
 
 -- | This data declaration is simply used as a token to distinguish
 --   the cairo backend: (1) when calling functions where the type
---   inference engine would otherwise have know way to know which
+--   inference engine would otherwise have no way to know which
 --   backend you wanted to use, and (2) as an argument to the
 --   'Backend' and 'Renderable' type classes.
 data Cairo = Cairo
@@ -64,15 +71,17 @@ data Cairo = Cairo
 
 -- | Output types supported by cairo, including four different file
 --   types (PNG, PS, PDF, SVG).  If you want to output directly to GTK
---   windows, see the diagrams-gtk package.
+--   windows, see the @diagrams-gtk@ package.
 data OutputType =
-    PNG      -- ^ Portable Network Graphics output.
-  | PS       -- ^ PostScript output
-  | PDF      -- ^ Portable Document Format output.
-  | SVG      -- ^ Scalable Vector Graphics output.
+    PNG         -- ^ Portable Network Graphics output.
+  | PS          -- ^ PostScript output
+  | PDF         -- ^ Portable Document Format output.
+  | SVG         -- ^ Scalable Vector Graphics output.
   | RenderOnly  -- ^ Don't output any file; the returned @IO ()@
-                -- action will do nothing, but the @Render ()@ action
-                -- can be used (e.g. to draw to a Gtk window)
+                --   action will do nothing, but the @Render ()@
+                --   action can be used (/e.g./ to draw to a Gtk
+                --   window; see the @diagrams-gtk@ package).
+  deriving (Eq, Ord, Read, Show, Bounded, Enum, Typeable)
 
 instance Monoid (Render Cairo R2) where
   mempty  = C $ return ()
@@ -80,17 +89,18 @@ instance Monoid (Render Cairo R2) where
 
 -- | The custom monad in which intermediate drawing options take
 --   place; 'Graphics.Rendering.Cairo.Render' is cairo's own rendering
---   monad.  At one point @RenderM@ really did use @StateT@, but then
---   the state got taken out... but the @StateT@ remains, now with a
---   zen-like state of type unit, \"just in case\".  Think of it as a
---   good luck charm.
-type RenderM a = StateT () C.Render a  -- no state for now
+--   monad.  Right now we simply maintain a Bool state to track
+--   whether or not we saw any lines in the most recent path (as
+--   opposed to loops).  If we did, we should ignore any fill
+--   attribute.  diagrams-lib separates lines and loops into separate
+--   path primitives so we don't have to worry about seeing them
+--   together in the same path.
+type RenderM a = StateT Bool C.Render a  -- no state for now
 
--- simple, stupid implementations of save and restore for now, since
--- it suffices to just reset the text alignment to "centered" on
--- restore.  But if need be we can switch to a more sophisticated
--- implementation using an "undoable state" monad which lets you save
--- (push state onto a stack) and restore (pop from the stack).
+-- Simple, stupid implementations of save and restore for now.  If
+-- need be we could switch to a more sophisticated implementation
+-- using an "undoable state" monad which lets you save (push state
+-- onto a stack) and restore (pop from the stack).
 
 -- | Push the current context onto a stack.
 save :: RenderM ()
@@ -109,19 +119,22 @@ instance Backend Cairo R2 where
           , cairoOutputType :: OutputType -- ^ the output format and associated options
           , cairoBypassAdjust  :: Bool    -- ^ Should the 'adjustDia' step be bypassed during rendering?
           }
+    deriving Show
 
   withStyle _ s t (C r) = C $ do
     save
     cairoMiscStyle s
+    put False
     r
+    ignoreFill <- get
     lift $ do
       cairoTransf t
-      cairoStrokeStyle s
+      cairoStrokeStyle ignoreFill s
       C.stroke
     restore
 
   doRender _ (CairoOptions file size out _) (C r) = (renderIO, r')
-    where r' = evalStateT r ()
+    where r' = evalStateT r False
           renderIO = do
             let surfaceF s = C.renderWith s r'
 
@@ -152,6 +165,7 @@ instance Backend Cairo R2 where
                                           c opts (d # reflectY)
     where setCairoSizeSpec sz o = o { cairoSizeSpec = sz }
 
+-- | Render an object that the cairo backend knows how to render.
 renderC :: (Renderable a Cairo, V a ~ R2) => a -> RenderM ()
 renderC a = case (render Cairo a) of C r -> r
 
@@ -189,10 +203,10 @@ fromFontWeight FontWeightNormal = C.FontWeightNormal
 fromFontWeight FontWeightBold   = C.FontWeightBold
 
 -- | Handle style attributes having to do with stroke.
-cairoStrokeStyle :: Style v -> C.Render ()
-cairoStrokeStyle s =
+cairoStrokeStyle :: Bool -> Style v -> C.Render ()
+cairoStrokeStyle ignoreFill s =
   sequence_
-  . catMaybes $ [ handle fColor
+  . catMaybes $ [ if ignoreFill then Nothing else handle fColor
                 , handle lColor  -- see Note [color order]
                 , handle lWidth
                 , handle lCap
@@ -209,9 +223,10 @@ cairoStrokeStyle s =
         lDashing (getDashing -> Dashing ds offs) =
           C.setDash ds offs
 
+-- | Set the source color.
 setSource :: Color c => c -> Style v -> C.Render ()
 setSource c s = C.setSourceRGBA r g b a'
-  where (r,g,b,a) = colorToRGBA c
+  where (r,g,b,a) = colorToSRGBA c
         a'        = case getOpacity <$> getAttr s of
                       Nothing -> a
                       Just d  -> a * d
@@ -247,21 +262,27 @@ fromFillRule :: FillRule -> C.FillRule
 fromFillRule Winding = C.FillRuleWinding
 fromFillRule EvenOdd = C.FillRuleEvenOdd
 
-instance Renderable (Segment R2) Cairo where
-  render _ (Linear v) = C . lift $ uncurry C.relLineTo (unr2 v)
+instance Renderable (Segment Closed R2) Cairo where
+  render _ (Linear (OffsetClosed v)) = C . lift $ uncurry C.relLineTo (unr2 v)
   render _ (Cubic (unr2 -> (x1,y1))
                   (unr2 -> (x2,y2))
-                  (unr2 -> (x3,y3)))
+                  (OffsetClosed (unr2 -> (x3,y3))))
     = C . lift $ C.relCurveTo x1 y1 x2 y2 x3 y3
 
 instance Renderable (Trail R2) Cairo where
-  render _ (Trail segs c) = C $ do
-    mapM_ renderC segs
-    lift $ when c C.closePath
+  render _ t = flip withLine t $ renderT . lineSegments
+    where
+      renderT segs =
+        C $ do
+          mapM_ renderC segs
+          lift $ when (isLoop t) C.closePath
+
+          when (isLine t) (put True)
+            -- remember that we saw a Line, so we will ignore fill attribute
 
 instance Renderable (Path R2) Cairo where
   render _ (Path trs) = C $ lift C.newPath >> F.mapM_ renderTrail trs
-    where renderTrail (unp2 -> p, tr) = do
+    where renderTrail (viewLoc -> (unp2 -> p, tr)) = do
             lift $ uncurry C.moveTo p
             renderC tr
 
